#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "logger.h"
#include "arbomb_utils.h"
#include "ziptest.h"

#define _ZT_EXIT_ENDREACHED 10

static int ztDEBUG = 0;

int ZT_set_debug( int level )
{
	ztDEBUG = level;
	return 0;
}


/*------------------------------------------------------------------------
 Procedure:     ZT_iszip ID:1
 Purpose:       Determins if a given filename contains a ZIP file.
 Input:
 Output:
 Errors:
------------------------------------------------------------------------*/
int ZT_iszip( char *fname )
{
	FILE *f;
	int result = 0;
	unsigned char buffer[4];

	// Open and test the zip file.
	f = fopen(fname,"r");
	if (!f)
	{
		LOGGER_log("ZT_iszip: Error opening file %s %s",fname,strerror(errno));
		return -1;
	}

	result = fread(buffer,1,4,f);

	if (result != 4)
	{
		LOGGER_log("ZT_iszip: Error, unable to read sufficient data to determin ZIP format.");
		result = 0;
	}

	if ((buffer[0] == 'P')&&(buffer[1] == 'K')&&(buffer[2] == 3)&&(buffer[3] == 4))
	{
		result = 1;
	}

	fclose(f);

	return result;
}



/*------------------------------------------------------------------------
 Procedure:     ZT_header_read ID:1
 Purpose:       Read the header for a single file in the ZIP archive
 Input:
 Output:
 Errors:
------------------------------------------------------------------------*/
int ZT_header_read( FILE *f, struct zipheader *zh )
{
	int result = 0;
	unsigned int value;

	// Read in the header, minus the two char pointers
	//
	value = 30;
	result = fread( zh, value, 1, f);

	if ((zh->sig[0] != 'P')||(zh->sig[1] != 'K'))
	{
		LOGGER_log("ERROR - Signature wrong, read in 0x%x %x %x %x\n",zh->sig[0],zh->sig[1],zh->sig[2],zh->sig[3]);
		return -1;
	}
	if (AB_u32int(zh->sig) == _ZS_CENTRAL_SIG)
	{
		if (ztDEBUG) LOGGER_log("Central reached, exit.\n");
		return _ZT_EXIT_ENDREACHED;
	}

	// Get the filename...
	value = AB_u16int(zh->filenamelen);
	zh->filename = malloc(value+1 *sizeof(char));
	result = fread( zh->filename, sizeof(char), value, f);
	if (result != value)
	{
		LOGGER_log("ZT_header_head: Error reading filename from ZIP headers.\n");
		return -1;
	}
	zh->filename[value] = '\0';

	// Reading past the data...
	value = (unsigned int)AB_u16int(zh->extrafieldlen) +AB_u32int(zh->sizecomp);
	if (ztDEBUG) LOGGER_log("Seeking +%d (EFL=%d+COMPSZ=%d)bytes...\n"\
		,value\
		,AB_u16int(zh->extrafieldlen)\
		,AB_u32int(zh->sizecomp)\
		);

	result = fseek(f, value, SEEK_CUR);
	if (result == -1)
	{
		LOGGER_log("ZT_header_read: Error seeking in file : %s",strerror(errno));
		return -1;
	}

	/*
	if (zh->gpbitflag && 0x04)
	{
		if (ztDEBUG) LOGGER_log("Seeking data descriptor...+%d\n",sizeof(struct datadescriptor));
		result = fseek(f, sizeof(struct datadescriptor), SEEK_CUR);
	}
	*/

	return result;
}


/*------------------------------------------------------------------------
 Procedure:     ZT_test ID:1
 Purpose:       Test a ZIP file against the specified limits as supplied.
 Input:
 Output:
 Errors:
------------------------------------------------------------------------*/
int ZT_test( char *fname, struct arbomb_limits *abl, struct arbomb_limits *lcs )
{
	int result = 0;
	struct zipheader zhreal, *zh;
	FILE *f;

	zh = &zhreal;

	lcs->archive_filecount = 0;
	lcs->single_csize = 0;
	lcs->single_ucsize = 0;

	// Open and test the zip file.
	f = fopen(fname,"r");
	if (!f)
	{
		LOGGER_log("ZT_test: Error opening file %s %s",fname,strerror(errno));
		return -1;
	}

	// While we still can read filenames/details of the ZIP file
	while (f)
	{
		result = ZT_header_read(f, zh);

		if (result == _ZT_EXIT_ENDREACHED)
		{
			result = 0;
		   break;
		}
		else if (result == -1)
		{
			result = AB_ERR_INTERNAL;
			break;
		}

		lcs->archive_filecount++;
		lcs->single_csize = AB_u32int(zh->sizecomp);
		lcs->single_ucsize = AB_u32int(zh->sizeuncomp);
		snprintf(lcs->filename,1023,"%s",zh->filename);

		// Prevent divide-by-zero errors
		if (lcs->single_csize < 1)
		{
			lcs->single_ratio = 0;
		}
		else
		{
			lcs->single_ratio = (double) lcs->single_ucsize / lcs->single_csize;
		}


		if (ztDEBUG)
		{
			fprintf(stdout,"ziptest: Filename = %s, Compressed Size = %d, Uncompressed size = %d, Ratio = %f\n"\
			, zh->filename\
			, lcs->single_csize\
			, lcs->single_ucsize\
			, lcs->single_ratio\
			);
		}

		// Test the results to see if we have any violations
		result = AB_limits_test( abl, lcs );

		// If the result is non-zero, then we must exit.
		if (result) break;

	} // While

	fclose(f);

	return result;
}

